# -*- coding: utf-8 -*-
r"""
IPywidgets with simple additional features to serve as singleton units to larger widgets.

AUTHORS ::

    Odile Bénassy, Nicolas Thiéry

"""
from traitlets import HasTraits, Int, Unicode
from ipywidgets import Button, Combobox, Dropdown, HTMLMath, Text, Textarea, ToggleButton, register


class Singleton(HasTraits):
    """Additional features to an ipywidgets widget."""
    _focus = Unicode().tag(sync=True)
    _tooltip = Unicode('').tag(sync=True) # set '' as default value
    tabindex = Int().tag(sync=True)

    def set_tooltip(self, s=''):
        self._tooltip = s

    def focus(self):
        self._focus = ''
        self._focus = 'on'

    def blur(self):
        self._focus = ''
        self._focus = 'off'

    def set_tabindex(self, i=0):
        self.tabindex = i

    def allow_focus(self):
        self.set_tabindex(0)

    def disallow_focus(self):
        self.set_tabindex(-1)


@register
class ButtonSingleton(Button, Singleton):
    """Button with tooltip and focus."""
    _model_name = Unicode('ButtonSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('ButtonSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)


@register
class ComboboxSingleton(Combobox, Singleton):
    """Combobox with tooltip and focus."""
    _model_name = Unicode('ComboboxSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('ComboboxSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)


@register
class DropdownSingleton(Dropdown, Singleton):
    """Dropdown with tooltip and focus."""
    _model_name = Unicode('DropdownSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('DropdownSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)


@register
class HTMLMathSingleton(HTMLMath, Singleton):
    """HTML Math widget with tooltip and focus."""
    _model_name = Unicode('HTMLMathSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('HTMLMathSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)


@register
class TextSingleton(Text, Singleton):
    """Input text with tooltip and focus."""
    _model_name = Unicode('TextSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('TextSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)


@register
class TextareaSingleton(Textarea, Singleton):
    """Text area with tooltip and focus."""
    _model_name = Unicode('TextareaSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('TextareaSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)


@register
class ToggleButtonSingleton(ToggleButton, Singleton):
    """Toggle button with tooltip and focus."""
    _model_name = Unicode('ToggleButtonSingletonModel').tag(sync=True)
    _model_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _model_module_version = Unicode('^0.7.6').tag(sync=True)
    _view_name = Unicode('ToggleButtonSingletonView').tag(sync=True)
    _view_module = Unicode('sage-combinat-widgets').tag(sync=True)
    _view_module_version = Unicode('^0.7.6').tag(sync=True)



# --*- encoding: utf-8 -*--
## This file is adapted from projects
## jupyter-widgets/widget-ts-cookiecutter
## and sagemath/sage_sample

# Copyright (c) Jupyter Development Team.
# Copyright (c) Sagemath Development Team.
# Distributed under the terms of the Modified BSD License.

from os.path import join as pjoin
from codecs import open # To open the README file with proper encoding
from setuptools.command.test import test as TestCommand # for tests

from setupbase import (
    create_cmdclass, install_npm, ensure_targets,
    find_packages, combine_commands,
    HERE
)

from setuptools import setup

# Get information from separate files (README, VERSION)
def readfile(filename):
    with open(filename,  encoding='utf-8') as f:
        return f.read()

# For the tests
class SageTest(TestCommand):
    def run_tests(self):
        errno = os.system("sage -t --force-lib sage_widget_adapters")
        if errno != 0:
            sys.exit(1)
        errno = os.system("sage -t --force-lib sage_combinat_widgets")
        if errno != 0:
            sys.exit(1)

# The name of the project
name = 'sage-combinat-widgets'
pyname = 'sage_combinat_widgets' # basis for JS extension

js_path = pjoin(HERE, 'js')
nb_path = pjoin(HERE, pyname, 'nbextension', 'static')
lab_path = pjoin(HERE, pyname, 'labextension')

# Representative files that should exist after a successful build
jstargets = [
    pjoin(nb_path, 'index.js'),
    pjoin(HERE, 'js', 'lib', 'extension.js'),
]

package_data_spec = {
    name: [
        'nbextension/static/*.*js*',
        'nbextension/static/*.*css',
        'labextension/*.tgz'
    ]
}

data_files_spec = [
    ('share/jupyter/nbextensions/sage-combinat-widgets',
        nb_path, '*.js*'),
    ('share/jupyter/nbextensions/sage-combinat-widgets',
        nb_path, '*.css'),
    ('share/jupyter/lab/extensions', lab_path, '*.tgz'),
    ('etc/jupyter/nbconfig/notebook.d', HERE, 'sage-combinat-widgets.json')
]


cmdclass = create_cmdclass('jsdeps', package_data_spec=package_data_spec,
                           data_files_spec=data_files_spec)
cmdclass['jsdeps'] = combine_commands(
    install_npm(pjoin(HERE, 'js'), build_cmd='build:all'),
    ensure_targets(jstargets),
)
cmdclass['test'] = SageTest

setup_args = dict(
    name            = name,
    version         = readfile("VERSION"),
    description     = 'Jupyter widgets for SAGE Combinat',
    long_description = readfile("README.rst"),
    cmdclass        = cmdclass,
    packages        = find_packages(),
    url             = 'https://github.com/sagemath/sage-combinat-widgets',
    author          = 'Odile Bénassy, Henri Derycke, Nicolas M. Thiéry',
    author_email    = 'odile.benassy@u-psud.fr',
    license         = 'GPLv3+',
    platforms       = "Linux, Mac OS X, Windows",
    keywords        = ['Jupyter', 'Widgets', 'SageMath', 'combinatorics'],
    classifiers     = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Mathematics',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Jupyter',
    ],
    include_package_data = True,
    install_requires = [
        'ipywidgets>=7.5.0',
    ],
    extras_require = {
        'docs': [
            'sage-package',
            'sphinx>=1.5',
        ],
    },
)

if __name__ == "__main__":
    setup(**setup_args)



from sage.combinat.tableau import Tableaux
from sage.combinat.skew_tableau import SkewTableau, SkewTableaux

Tableaux.options.display="array"
Tableaux.options.convention="French"


class JeuDeTaquin(SkewTableau):
    def __init__(self, st):
        r"""
        sage: from sage.combinat.skew_tableau import SkewTableau
        sage: from jeudetaquin import JeuDeTaquin
        sage: jdt = JeuDeTaquin([[None,2,3],[None,4],[5]]) ; jdt
          5
          .  4
          .  2  3
        sage: jdt.done(), jdt.has_hole()
        (False, False)

        sage: jdt.create_hole((0,0))
        Traceback (most recent call last):
        ...
        ValueError: corner must be an inner corner

        sage: jdt.create_hole((1,0))
        sage: jdt
          5
          *  4
          .  2  3
        sage: jdt.done(), jdt.has_hole()
        (False, True)
        sage: jdt.slide(); jdt
          5
          4
          .  2  3
        sage: jdt.done(), jdt.has_hole()
        (False, False)

        sage: jdt.slide()
        Traceback (most recent call last):
        ...
        ValueError: There is no hole

        sage: jdt.create_hole((0,0)); jdt
          5
          4
          *  2  3
        sage: jdt.done(), jdt.has_hole()
        (False, True)
        sage: jdt.slide(); jdt
          5
          4
          2  *  3
        sage: jdt.done(), jdt.has_hole()
        (False, True)

        sage: jdt.create_hole((0,1))
        Traceback (most recent call last):
        ...
        ValueError: There is already a hole at (0, 1)

        sage: jdt.slide(); jdt
          5
          4
          2  3
        sage: jdt.done(), jdt.has_hole()
        (True, False)

        sage: t = SkewTableau([[None,None,None,None,2,3,7,8],[None,None,4,9],[None,5,6],[10]])
        sage: jdt = JeuDeTaquin(t); jdt
         10
          .  5  6
          .  .  4  9
          .  .  .  .  2  3  7  8
        sage: jdt.create_hole((0, 3)); jdt
         10
          .  5  6
          .  .  4  9
          .  .  .  *  2  3  7  8
        sage: jdt.slide(); jdt
         10
          .  5  6
          .  .  4  9
          .  .  .  2  *  3  7  8
        sage: jdt.slide(); jdt
         10
          .  5  6
          .  .  4  9
          .  .  .  2  3  *  7  8
        sage: jdt.slide(); jdt
         10
          .  5  6
          .  .  4  9
          .  .  .  2  3  7  *  8
        sage: jdt.slide(); jdt
         10
          .  5  6
          .  .  4  9
          .  .  .  2  3  7  8
        sage: jdt.create_hole((1, 1)); jdt
         10
          .  5  6
          .  *  4  9
          .  .  .  2  3  7  8
        sage: jdt.slide(); jdt
         10
          .  5  6
          .  4  *  9
          .  .  .  2  3  7  8
        sage: jdt.slide(); jdt
         10
          .  5
          .  4  6  9
          .  .  .  2  3  7  8

        sage: jdt.create_hole((2, 0)); jdt
         10
          *  5
          .  4  6  9
          .  .  .  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  .  .  2  3  7  8

        sage: jdt.create_hole((0, 2)); jdt
         10
          5
          .  4  6  9
          .  .  *  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  .  2  *  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  .  2  3  *  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  .  2  3  7  *  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  .  2  3  7  8

        sage: jdt.create_hole((0, 1)); jdt
         10
          5
          .  4  6  9
          .  *  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  2  *  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  2  3  *  7  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  2  3  7  *  8
        sage: jdt.slide(); jdt
         10
          5
          .  4  6  9
          .  2  3  7  8

        sage: jdt.create_hole((1, 0)); jdt
         10
          5
          *  4  6  9
          .  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          4  *  6  9
          .  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          4  6  *  9
          .  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          4  6  9
          .  2  3  7  8

        sage: jdt.create_hole((0, 0)); jdt
         10
          5
          4  6  9
          *  2  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          4  6  9
          2  *  3  7  8
        sage: jdt.slide(); jdt
         10
          5
          4  6  9
          2  3  *  7  8
        sage: jdt.slide(); jdt
         10
          5
          4  6  9
          2  3  7  *  8
        sage: jdt.slide(); jdt
         10
          5
          4  6  9
          2  3  7  8
        """
        super(JeuDeTaquin, self).__init__(SkewTableaux(), st)
        self._hole = None
        self._new_st = None

    def _repr_(self):
        if self._hole is None:
            lst = self.to_list()
        else:
            lst = self._new_st
        none_str = lambda x: "  ." if x is None else "%3s"%str(x)
        if self.parent().options('convention') == "French":
            new_rows = ["".join(map(none_str, row)) for row in reversed(lst)]
        else:
            new_rows = ["".join(map(none_str, row)) for row in lst]
        return '\n'.join(new_rows)

    def __eq__(self, obj):
        if obj._hole != self._hole:
            return False
        if obj._new_st != self._new_st:
            return False
        return super(JeuDeTaquin, self) == super(JeuDeTaquin, obj)

    def done(self):
        return self._hole is None and not self.inner_shape()

    def has_hole(self):
        return self._hole is not None

    def create_hole(self, corner):
        if self._hole is not None:
            raise ValueError("There is already a hole at %s" % (self._hole,))
        inner_corners = self.inner_shape().corners()
        if tuple(corner) not in inner_corners:
            raise ValueError("corner must be an inner corner")
        self._hole = corner
        self._new_st = self.to_list()
        spotl, spotc = self._hole
        self._new_st[spotl][spotc] = True

    def slide(self):
        if self._hole is None:
            raise ValueError("There is no hole")
        spotl, spotc = self._hole
        #Check to see if there is nothing to the right
        if spotc == len(self._new_st[spotl]) - 1:
            #Swap the hole with the cell below
            self._new_st[spotl][spotc] = self._new_st[spotl+1][spotc]
            self._new_st[spotl+1][spotc] = -1
            spotl += 1

        #Check to see if there is nothing below
        elif (spotl == len(self._new_st) - 1 or
              len(self._new_st[spotl+1]) <= spotc):
            #Swap the hole with the cell to the right
            self._new_st[spotl][spotc] = self._new_st[spotl][spotc+1]
            self._new_st[spotl][spotc+1] = -1
            spotc += 1

        else:
        #If we get to this stage, we need to compare
            below = self._new_st[spotl+1][spotc]
            right = self._new_st[spotl][spotc+1]
            if below <= right:
                #Swap with the cell below
                self._new_st[spotl][spotc] = self._new_st[spotl+1][spotc]
                self._new_st[spotl+1][spotc] = -1
                spotl += 1
            else:
                #Otherwise swap with the cell to the right
                self._new_st[spotl][spotc] = self._new_st[spotl][spotc+1]
                self._new_st[spotl][spotc+1] = -1
                spotc += 1
        if (spotl, spotc) in self.outer_shape().corners():
            self._new_st[spotl].pop()
            if not self._new_st[spotl]:
                self._new_st.pop()
            self.__init__(self._new_st) # Resurrection?
        else:
            self._hole = (spotl, spotc)



# -*- coding: utf-8 -*-
r"""
Grid View Adapter for grid-representable graphs

**Grid View graphs operations:**

.. csv-table::
    :class: contentstable
    :widths: 30, 70
    :delim: |

    :meth:`~GraphGridViewAdapter.cell_to_display` | Static method for typecasting cell content to widget display value
    :meth:`~GraphGridViewAdapter.display_to_cell` | Instance method for typecasting widget display value to cell content
    :meth:`~GraphGridViewAdapter.compute_cells` | Compute graph cells as a dictionary { coordinate pair : label }
    :meth:`~GraphGridViewAdapter.from_cells` | Create a new graph from a cells dictionary
    :meth:`~GraphGridViewAdapter.get_cell` | Get the graph cell content (i.e. None)
    :meth:`~GraphGridViewAdapter.addable_cells` | List addable cells
    :meth:`~GraphGridViewAdapter.removable_cells` | List removable cells
    :meth:`~GraphGridViewAdapter.add_cell` | Add a cell
    :meth:`~GraphGridViewAdapter.remove_cell` | Remove a cell
    :meth:`~GraphGridViewAdapter.append_row` | Append a row
    :meth:`~GraphGridViewAdapter.remove_row` | Remove a row at given index
    :meth:`~GraphGridViewAdapter.append_column` | Append a column
    :meth:`~GraphGridViewAdapter.remove_column` | Remove a column at given index

AUTHORS ::

    Odile Bénassy, Nicolas Thiéry

"""
from sage.graphs.graph import Graph
from sage_widget_adapters.generic_grid_view_adapter import GridViewAdapter
from six import text_type

class GraphGridViewAdapter(GridViewAdapter):
    r"""
    Grid view adapter for grid-representable graphs.

    ATTRIBUTES::
        * ``objclass`` -- Graph
        * ``celltype`` -- bool
        * ``cellzero`` -- False
    """
    objclass = Graph
    celltype = bool
    cellzero = False

    @staticmethod
    def cell_to_display(cell_content, display_type=bool):
        r"""
        From object cell content
        to widget display value.

        TESTS ::

            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: GraphGridViewAdapter.cell_to_display(True)
            True
            sage: from six import text_type
            sage: GraphGridViewAdapter.cell_to_display("my string", text_type)
            ''
        """
        if display_type == text_type:
            return ''
        elif cell_content:
            return cell_content
        elif display_type == bool:
            return False

    def display_to_cell(self, display_value, display_type=bool):
        r"""
        From widget cell value
        to object display content

        TESTS ::

            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: ga = GraphGridViewAdapter()
            sage: ga.display_to_cell(True)
            True
            sage: ga.display_to_cell('')
            False
        """
        if not display_value or display_type == text_type:
            return self.cellzero
        return display_value

    @staticmethod
    def compute_cells(obj):
        r"""
        From the graph vertices,
        make a dictionary { coordinates pair : None }

        TESTS ::

            sage: from sage.graphs.generators.families import AztecDiamondGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = AztecDiamondGraph(2)
            sage: GraphGridViewAdapter.compute_cells(g)
            {(0, 1): None,
            (0, 2): None,
            (1, 0): None,
            (1, 1): None,
            (1, 2): None,
            (1, 3): None,
            (2, 0): None,
            (2, 1): None,
            (2, 2): None,
            (2, 3): None,
            (3, 1): None,
            (3, 2): None}
        """
        cells = {}
        for v in obj.vertices():
            cells[v] = None
        return cells

    @classmethod
    def from_cells(cls, cells={}):
        r"""
        From a dictionary { coordinates pair : None }
        return a graph with one vertex for every coordinates pair

        TESTS ::

            sage: from sage.graphs.generators.families import AztecDiamondGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: GraphGridViewAdapter.from_cells({(0, 0): None, (0, 1): None, (1, 0): None, (1, 1): None, (2, 0): None, (2, 1): None})
            Graph on 6 vertices
        """
        g = Graph()
        g.add_vertices(list(cells.keys()))
        return cls.objclass(g)

    @staticmethod
    def get_cell(obj, pos):
        r"""
        From a graph `graph` and a tuple `pos`,
        return the object cell value at position `pos`.

        TESTS ::

            sage: from sage.graphs.generators.families import AztecDiamondGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = AztecDiamondGraph(2)
            sage: GraphGridViewAdapter.get_cell(g, (1,3)) is None
            True
        """
        return None

    @staticmethod
    def addable_cells(obj):
        r"""
        No cell should be added in isolation
        except for linear graphs

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((2,3))
            sage: GraphGridViewAdapter.addable_cells(g)
            []
            sage: g = GridGraph((1,3))
            sage: GraphGridViewAdapter.addable_cells(g)
            [(0, 3)]
        """
        if not obj.num_verts():
            return [(0,0)]
        row_max, col_max = 0, 0
        for t in obj.vertex_iterator():
            row_max = max(row_max, t[0])
            col_max = max(col_max, t[1])
            if row_max > 0 and col_max > 0:
                return []
        if row_max > 0:
            return [(row_max + 1, 0)]
        elif col_max > 0:
            return [(0, col_max + 1)]
        return [(0,1),(1,0)]

    @staticmethod
    def removable_cells(obj):
        r"""
        No cell should be removed in isolation
        except for linear graphs

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((2,3))
            sage: GraphGridViewAdapter.removable_cells(g)
            []
            sage: g = GridGraph((1,3))
            sage: GraphGridViewAdapter.removable_cells(g)
            [(0, 2)]
        """
        row_max, col_max = 0, 0
        for t in obj.vertex_iterator():
            row_max = max(row_max, t[0])
            col_max = max(col_max, t[1])
            if row_max > 0 and col_max > 0:
                return []
        if row_max > 0:
            return [(row_max, 0)]
        elif col_max > 0:
            return [(0, col_max)]
        return [(0,0)]

    def add_cell(self, obj, pos, val=None, dirty={}):
        r"""
        Add a cell to the graph.

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((1,2))
            sage: ga = GraphGridViewAdapter()
            sage: ga.add_cell(g, (0,2))
            Grid Graph for [1, 2]: Graph on 3 vertices
        """
        if not pos in self.addable_cells(obj):
            raise ValueError("Position '%s' is not addable." % str(pos))
        if pos in obj.vertices():
            raise ValueError("This cell (position=%s) is already in the graph." % str(pos))
        obj.add_vertex(pos)
        return obj

    def remove_cell(self, obj, pos, dirty={}):
        r"""
        Remove a cell from the graph.

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((1, 2))
            sage: ga = GraphGridViewAdapter()
            sage: ga.remove_cell(g, (0,1))
            Grid Graph for [1, 2]: Graph on 1 vertex
        """
        if not pos in self.removable_cells(obj):
            raise ValueError("Cell position '%s' is not removable." % str(pos))
        obj.delete_vertex(pos)
        return obj

    def append_row(self, obj):
        r"""
        Add a row to the graph.

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((3,2))
            sage: ga = GraphGridViewAdapter()
            sage: ga.append_row(g)
            Grid Graph for [3, 2]: Graph on 8 vertices
        """
        row_max, col_max = 0, 0
        for t in obj.vertex_iterator():
            row_max = max(row_max, t[0])
            col_max = max(col_max, t[1])
        obj.add_vertices([(row_max + 1, j) for j in range(col_max + 1)])
        return obj

    def remove_row(self, obj, index=None):
        r"""
        Remove a row from the graph

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((3,2))
            sage: ga = GraphGridViewAdapter()
            sage: ga.remove_row(g)
            Grid Graph for [3, 2]: Graph on 4 vertices
        """
        row_max, col_max = 0, 0
        for t in obj.vertex_iterator():
            row_max = max(row_max, t[0])
            col_max = max(col_max, t[1])
        obj.delete_vertices([(row_max, j) for j in range(col_max + 1)])
        return obj

    def append_column(self, obj):
        r"""
        Add a column to the graph.

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((3,2))
            sage: ga = GraphGridViewAdapter()
            sage: ga.append_column(g)
            Grid Graph for [3, 2]: Graph on 9 vertices
        """
        row_max, col_max = 0, 0
        for t in obj.vertex_iterator():
            row_max = max(row_max, t[0])
            col_max = max(col_max, t[1])
        obj.add_vertices([(i, col_max + 1) for i in range(row_max + 1)])
        return obj

    def remove_column(self, obj, index=None):
        r"""
        Remove a column from the graph

        TESTS ::

            sage: from sage.graphs.generators.basic import GridGraph
            sage: from sage_widget_adapters.graphs.graph_grid_view_adapter import GraphGridViewAdapter
            sage: g = GridGraph((3,2))
            sage: ga = GraphGridViewAdapter()
            sage: ga.remove_column(g)
            Grid Graph for [3, 2]: Graph on 3 vertices
        """
        row_max, col_max = 0, 0
        for t in obj.vertex_iterator():
            row_max = max(row_max, t[0])
            col_max = max(col_max, t[1])
        obj.delete_vertices([(i, col_max) for i in range(row_max + 1)])
        return obj
